/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.abstractschema;

import java.util.Collection;
import java.util.Iterator;

import org.tranql.schema.Association;
import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Entity;
import org.tranql.schema.FKAttribute;

/**
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public abstract class AbstractAssociationEnd implements AssociationEnd {
    private final String name;
    private final Entity entity;
    private final boolean isSingle;
    private final boolean isCascadeDelete;
    private final Association association;
    private final boolean isVirtual;
    private final boolean isOnPKSide;
    
    public AbstractAssociationEnd(String name, Entity entity, boolean isSingle, boolean isCascadeDelete, Association association, boolean isVirtual, boolean isOnPKSide) {
        this.name = name;
        this.entity = entity;
        this.isSingle = isSingle;
        this.isCascadeDelete = isCascadeDelete;
        this.association = association;
        this.isVirtual = isVirtual;
        this.isOnPKSide = isOnPKSide;
    }

    public String getName() {
        return name;
    }

    public Entity getEntity() {
        return entity;
    }

    public Association getAssociation() {
        return association;
    }

    public boolean isSingle() {
        return isSingle;
    }

    public boolean isMulti() {
        return !isSingle;
    }

    public boolean isCascadeDelete() {
        return isCascadeDelete;
    }
    
    public boolean isVirtual() {
        return isVirtual;
    }
    
    public boolean isOnPKSide() {
        return isOnPKSide;
    }
    
    public boolean isOneToOne() {
        return isSingle() && association.getOtherEnd(this).isSingle();
    }

    public boolean isOneToMany() {
        return isMulti() && association.getOtherEnd(this).isSingle();
    }
    
    public boolean isManyToOne() {
        return isSingle() && association.getOtherEnd(this).isMulti();
    }
    
    public boolean isManyToMany() {
        return isMulti() && association.getOtherEnd(this).isMulti();
    }
    
    public boolean hasFKAttribute(String fkAttribute) {
        if (isOnPKSide()) {
            return false;
        }
        
        Association.JoinDefinition joinDefinition = getAssociation().getJoinDefinition(); 
        Collection fkAttributes = joinDefinition.getPKToFKMapping().values();
        for (Iterator iterator = fkAttributes.iterator(); iterator.hasNext();) {
            FKAttribute currfkAttribute = (FKAttribute) iterator.next();
            if (currfkAttribute.getName().equals(fkAttribute)) {
                return true;
            }
        }
        
        return false;
    }
}
