/**
 *  Copyright 2006 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.builder;

import org.tranql.cache.CacheRow;
import org.tranql.cache.GlobalSchema;
import org.tranql.cache.InTxCache;
import org.tranql.ejb.EJBSchema;
import org.tranql.field.Row;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.query.SchemaMapper;
import org.tranql.query.UpdateCommand;
import org.tranql.sql.SQLSchema;

public class DynamicCommandBuilder extends StaticCommandBuilder {

    public DynamicCommandBuilder(String name,
            EJBSchema ejbSchema,
            SQLSchema sqlSchema,
            GlobalSchema globalSchema) {
        super(name, ejbSchema, sqlSchema, globalSchema);
    }
    
    public UpdateCommand buildStore() throws QueryException {
        DynamicSQLUpdateCommand command =
            new DynamicSQLUpdateCommand(name, ejbSchema, sqlSchema, globalSchema);
        return command;
    }
    
    private static class DynamicSQLUpdateCommand implements UpdateCommand {
        private final String name;
        private final EJBSchema ejbSchema;
        private final SQLSchema sqlSchema;
        private final GlobalSchema globalSchema;

        public DynamicSQLUpdateCommand(String name, EJBSchema ejbSchema, SQLSchema sqlSchema, GlobalSchema globalSchema) {
            this.name = name;
            this.ejbSchema = ejbSchema;
            this.sqlSchema = sqlSchema;
            this.globalSchema = globalSchema;
        }

        public Query getQuery() {
            throw new UnsupportedOperationException();
        }

        public int execute(InTxCache cache, Row row) throws QueryException {
            UpdateCommand command = buildUpdateCommand((CacheRow) row);
            return command.execute(cache, row);
        }
        
        private UpdateCommand buildUpdateCommand(CacheRow row) throws QueryException {
            IdentityDefinerBuilder idDefBuilder = new IdentityDefinerBuilder(ejbSchema, globalSchema);
            EJBQueryBuilder queryBuilder = new EJBQueryBuilder(idDefBuilder);
            UpdateCommand command = queryBuilder.buildStore(name, row);

            SchemaMapper mapper = new SchemaMapper(sqlSchema);
            return mapper.transform(command);
        }
    }
}
