/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import org.tranql.field.Row;
import org.tranql.field.FieldTransformException;
import org.tranql.ql.QueryException;
import org.tranql.query.ResultHandler;

/**
 *
 *
 * @version $Revision: 188 $ $Date: 2005-07-03 06:07:58 -0400 (Sun, 03 Jul 2005) $
 */
public class CacheRowLoader implements ResultHandler {
    private final EmptySlotLoader[] accessors;
    private boolean fetched = false;

    public CacheRowLoader(EmptySlotLoader[] accessors) {
        this.accessors = accessors;
    }

    public Object fetched(Row row, Object arg) throws QueryException {
        CacheRow cacheRow = (CacheRow) arg;
        try {
            for (int i=0; i < accessors.length; i++) {
                accessors[i].set(cacheRow, row);
            }
        } catch (FieldTransformException e) {
            throw new QueryException(e);
        }
        this.fetched = true;
        return arg;
    }

    public Object endFetched(Object arg) throws QueryException {
        return arg;
    }
    
    public boolean wasFetched() {
        return fetched;
    }
}
