/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.HashMap;

import org.tranql.identity.GlobalIdentity;
import org.tranql.ql.QueryException;

/**
 *
 *
 *
 * @version $Revision: 276 $ $Date: 2005-10-01 04:41:03 -0400 (Sat, 01 Oct 2005) $
 */
public class InTxCache {
    private final CacheFlushStrategy flushStrategy;
    private final Map cache = new HashMap();
    private final Collection newIDSnapshot = new ArrayList();

    public InTxCache(CacheFlushStrategy flushStrategy) {
        this.flushStrategy = flushStrategy;
    }

    public void associate(CacheRow row) throws AlreadyAssociatedException {
        GlobalIdentity id = row.getId();
        Map tableMap = getTableMap(id);
        CacheRow previousRow = (CacheRow) tableMap.get(id);
        if (null != previousRow) {
            if (previousRow.getState() != CacheRowState.REMOVED ||
                    row.getState() != CacheRowState.NEW) {
                throw new AlreadyAssociatedException();
            }
            if ( false == newIDSnapshot.contains(id) ) {
                row.mergeWithRemoved(previousRow);
            }
        }
        if (row.getState() == CacheRowState.NEW) {
            newIDSnapshot.add(id);
        }
        tableMap.put(id, row);
        flushStrategy.rowAssociated(row);
    }

    public CacheRow get(GlobalIdentity id) {
        Map tableMap = (Map) cache.get(id.getTable());
        if (tableMap == null) {
            return null;
        } else {
            return (CacheRow) tableMap.get(id);
        }
    }

    public void add(CacheRow row) throws AlreadyAssociatedException {
        associate(row);
        flushStrategy.rowAdded(row);
    }

    public void modified(CacheRow row) {
        flushStrategy.rowModified(row);
    }

    public void remove(CacheRow row) {
        flushStrategy.rowRemoved(row);
    }

    public void flush() throws QueryException {
        newIDSnapshot.clear();
        flushStrategy.flush(cache);
    }

    private Map getTableMap(GlobalIdentity id) {
        CacheTable table = id.getTable();
        Map tableMap = (Map) cache.get(table);
        if (tableMap == null) {
            tableMap = new HashMap();
            cache.put(table, tableMap);
        }
        return tableMap;
    }

    Map getCacheMap() {
        return cache;
    }
}
