/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ddl;

/**
 *
 * @version $Revision: 246 $ $Date: 2005-08-26 22:09:42 -0400 (Fri, 26 Aug 2005) $
 */
public abstract class AbstractNode implements Node {
    private Node sibling;
    private Node child;
    
    public Node getSibling() {
        return sibling;
    }

    public void setSibling(Node sibling) {
        assert this.sibling == null: "current sibling should be null";
        this.sibling = sibling;
    }

    public Node getChild() {
        return child;
    }

    public Node addChild(Node child) {
        if (child == null) {
            throw new IllegalArgumentException("child is null");
        }
        
        if (this.child == null) {
            this.child = child;
        } else {
            Node sib = this.child;
            while (sib.getSibling() != null) {
                sib = sib.getSibling();
            }
            sib.setSibling(child);
        }
        return this;
    }

    public boolean isOnlyChild() {
        return child != null && child.getSibling() == null;
    }
    
}
