/**
 *  Copyright 2006 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.dynamicquery;

import java.util.ArrayList;
import java.util.List;

import org.tranql.field.FieldTransform;
import org.tranql.field.Row;
import org.tranql.ql.QueryException;
import org.tranql.query.CollectionResultHandler;
import org.tranql.query.ObjectResultHandler;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;
import org.tranql.sql.EJBQLQueryToSQLCommandCompiler;

/**
 * 
 * @version $Revision: 327 $ $Date: 2006-03-02 06:04:50 -0500 (Thu, 02 Mar 2006) $
 */
public class BasicQuery implements Query {
    private final Row params;
    private final QueryCommand command;
    private FieldTransform resultAccessor;
    
    public BasicQuery(EJBQLQueryToSQLCommandCompiler queryBuilder, String ejbQL, Class[] types) {
        DynamicEJBQLQuery query = new DynamicEJBQLQuery(types, ejbQL);
        try {
            command = queryBuilder.buildQueryCommand(query);
        } catch (QueryException e) {
            throw (IllegalStateException) new IllegalStateException().initCause(e);
        }
        resultAccessor = command.getQuery().getResultAccessors()[0];
        params = new Row(new Object[types.length]);
    }

    public List getResultList() {
        CollectionResultHandler handler = new CollectionResultHandler(resultAccessor);
        List resultList = new ArrayList();
        executeQuery(handler, resultList);
        return resultList;
    }

    public Object getSingleResult() {
        ObjectResultHandler handler = new ObjectResultHandler(resultAccessor);
        Object result = executeQuery(handler, null);
        return result;
    }

    public Query setParameter(int index, Object value) {
        if (index < 0 || index + 1 > params.getValues().length) {
            throw new IllegalArgumentException(
                    "Index " + index + 
                    " is outside the parameter range: 0 to " + 
                    (params.getValues().length - 1));
        }
        params.set(index, value);
        return this;
    }

    public Query setParameters(Object[] values) {
        Object[] paramValues = params.getValues();
        if (null == values) {
            throw new IllegalArgumentException("Provided parameters must not be null.");
        } else if (paramValues.length != values.length) {
            throw new IllegalArgumentException(paramValues.length + 
                    " parameters are expected.");
        }
        System.arraycopy(values, 0, paramValues, 0, paramValues.length);
        return this;
    }
    
    private Object executeQuery(ResultHandler handler, Object opaque) {
        try {
            return command.execute(null, handler, params, opaque);
        } catch (QueryException e) {
            throw (IllegalStateException) new IllegalStateException().initCause(e);
        } finally {
            Object[] paramValues = params.getValues();
            for (int i = 0; i < paramValues.length; i++) {
                paramValues[i] = null;
            }
        }
    }
}
