/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import javax.ejb.EJBException;

import org.tranql.cache.CacheRow;
import org.tranql.cache.FaultHandler;
import org.tranql.cache.InTxCache;
import org.tranql.ql.QueryException;

/**
 *
 *
 *
 * @version $Revision: 276 $ $Date: 2005-10-01 04:41:03 -0400 (Sat, 01 Oct 2005) $
 */
public class CMPFieldFaultTransform implements CMPFieldTransform {
    private final CMPFieldTransform next;
    private final FaultHandler handler;
    private final int[] slots;

    public CMPFieldFaultTransform(CMPFieldTransform next, FaultHandler handler, int[] slots) {
        this.next = next;
        this.handler = handler;
        this.slots = slots;
    }

    public Object get(InTxCache cache, CacheRow row) {
        try {
            boolean fault = false;
            for (int i = 0; i < slots.length; i++) {
                int slot = slots[i];
                if (!row.isLoaded(slot)) {
                    fault = true;
                    break;
                }
            }
            if (fault) {
                handler.fieldFault(cache, row);
            }
            for (int i = 0; i < slots.length; i++) {
                int slot = slots[i];
                if (!row.isLoaded(slot)) {
                    // slot is still not loaded
                    // this is caused when the row no londer exists in the database
                    throw new NotLoadedException();
                }
            }
            return next.get(cache, row);
        } catch (QueryException e) {
            throw new EJBException("Unable to load data for field", e);
        }
    }

    public void set(InTxCache cache, CacheRow row, Object value) {
        next.set(cache, row, value);
    }
}
