/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import javax.ejb.EJBException;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheRowState;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityTransform;
import org.tranql.identity.IdentityTransformException;

/**
 *
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public class CMPMappedToCMRAccessor implements CMPFieldTransform {
    private final CMPFieldTransform cmrField;
    private final CMPFieldTransform cmpField;
    private final IdentityTransform transform;
    
    public CMPMappedToCMRAccessor(CMPFieldTransform cmrField, CMPFieldTransform cmpField, IdentityTransform transform) {
        this.cmrField = cmrField;
        this.cmpField = cmpField;
        this.transform = transform;
    }

    public Object get(InTxCache cache, CacheRow row) {
        GlobalIdentity id = (GlobalIdentity) cmrField.get(cache, row);
        try {
            return transform.getDomainIdentity(id);
        } catch (IdentityTransformException e) {
            throw new EJBException("Cannot extract domain identity", e);
        }
    }

    public void set(InTxCache cache, CacheRow row, Object value) {
        GlobalIdentity id;
        try {
            id = transform.getGlobalIdentity(value);
        } catch (IdentityTransformException e) {
            throw new EJBException("Cannot extract global identity", e);
        }
        cmpField.set(cache, row, value);
        cmrField.set(cache, row, id);
    }
}
