/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import javax.ejb.EJBLocalObject;

import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityTransform;
import org.tranql.identity.IdentityTransformException;

/**
 * Identity transform that converts between an EJBLocalObject reference
 * and a global id for the cache.
 *
 * @version $Revision: 57 $ $Date: 2004-04-21 17:45:17 -0400 (Wed, 21 Apr 2004) $
 */
public class LocalProxyTransform implements IdentityTransform {
    private final IdentityTransform next;
    private final EJBProxyFactory factory;

    public LocalProxyTransform(IdentityTransform next, EJBProxyFactory factory) {
        this.next = next;
        this.factory = factory;
    }

    public GlobalIdentity getGlobalIdentity(Object domainId) throws IdentityTransformException {
        Object pk = ((EJBLocalObject) domainId).getPrimaryKey();
        return next.getGlobalIdentity(pk);
    }

    public Object getDomainIdentity(GlobalIdentity globalId) throws IdentityTransformException {
        Object pk = next.getDomainIdentity(globalId);
        return factory.getEJBLocalObject(pk);
    }
}
