/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheRowLoader;
import org.tranql.cache.EmptySlotLoader;
import org.tranql.cache.FaultHandler;
import org.tranql.cache.InTxCache;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.ql.QueryException;
import org.tranql.query.QueryCommand;

/**
 *
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class SingleValuedCMRFaultHandler implements FaultHandler {
    private final QueryCommand command;
    private final IdentityDefiner idDefiner;
    private final EmptySlotLoader[] accessors;

    public SingleValuedCMRFaultHandler(QueryCommand command, IdentityDefiner idDefiner, EmptySlotLoader[] accessors) {
        this.command = command;
        this.idDefiner = idDefiner;
        this.accessors = accessors;

    }

    public void fieldFault(InTxCache cache, CacheRow row) throws QueryException {
        Row params = idDefiner.extractIdentity(row.getId());
        CacheRowLoader handler = new CacheRowLoader(accessors);
        command.execute(cache, handler, params, row);
        if (!handler.wasFetched()) {
            for (int i = 0; i < accessors.length; i++) {
                EmptySlotLoader accessor = accessors[i];
                accessor.setNull(row);
            }
        }
    }

    public void rowFault(InTxCache cache, GlobalIdentity id) throws QueryException {
        throw new UnsupportedOperationException();
    }
}
