/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import java.util.Stack;

import org.tranql.ql.Node;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySourceDictionary;
import org.tranql.ql.SubQuery;

/**
 * 
 * @version $Revision: 186 $ $Date: 2005-06-22 07:22:41 -0400 (Wed, 22 Jun 2005) $
 */
class AbstractSubQueryAwareTransformer extends InterRepresentationTransformer {
    
    public AbstractSubQueryAwareTransformer(QuerySourceDictionary sourceDictionary) {
        super(sourceDictionary);
    }

    public Object visit(Query node, Object param) throws QueryException {
        AbstractSubQueryContext ctx = (AbstractSubQueryContext) param;

        ctx.push(node);
        Object returned = super.visit(node, ctx);
        ctx.pop();
        return returned;
    }

    public Object visit(SubQuery node, Object param) throws QueryException {
        AbstractSubQueryContext ctx = (AbstractSubQueryContext) param;

        ctx.push(node);
        super.visit(node, ctx);
        ctx.pop();
        return param;
    }

    protected static class AbstractSubQueryContext {
        private final Stack nodes = new Stack();
        
        public void push(Node node) {
            nodes.push(node);
        }
        
        public Node pop() {
            return (Node) nodes.pop();
        }
        
        protected Node peek() {
            return (Node) nodes.peek();
        }
    }

}
