/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.tranql.builder.IdentityDefinerBuilder;
import org.tranql.cache.GlobalSchema;
import org.tranql.ejb.EJBSchema;
import org.tranql.ejbqlcompiler.WhereBracketAppender; 
import org.tranql.ejbqlcompiler.QuerySourceRegister;
import org.tranql.ejbqlcompiler.EmptyWhereAppender;
import org.tranql.ejbqlcompiler.DerivedTableToCrossJoinTransformer;
import org.tranql.ejbqlcompiler.PathTransformer;
import org.tranql.ejbqlcompiler.MemberOfTransformer;
import org.tranql.ejbqlcompiler.EmptyTransformer;
import org.tranql.ejbqlcompiler.IsNullTransformer;
import org.tranql.ejbqlcompiler.EntityReferenceTransformer;
import org.tranql.ejbqlcompiler.EmptyWhereTransformer;

import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySourceDictionary;
import org.tranql.query.CommandTransform;
import org.tranql.query.QueryCommand;
import org.tranql.query.SchemaMapper;
import org.tranql.sql.EJBQLCompilerFactory;
import org.tranql.sql.SQLSchema;


/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class DB2EJBQLCompilerFactory implements EJBQLCompilerFactory {

    public EJBQLCompiler factoryEJBQLCompiler(EJBSchema ejbSchema, SQLSchema sqlSchema, GlobalSchema globalSchema) {
        return new DB2EJBQLCompiler(ejbSchema, sqlSchema, globalSchema);
    }
    
    private static class DB2EJBQLCompiler implements EJBQLCompilerFactory.EJBQLCompiler {
        private final IdentityDefinerBuilder identityDefinerBuilder;
        
        public DB2EJBQLCompiler(EJBSchema ejbSchema, SQLSchema sqlSchema, GlobalSchema globalSchema) {
            identityDefinerBuilder = new IdentityDefinerBuilder(ejbSchema, globalSchema);
        }

        public QueryCommand transform(QueryCommand command) throws QueryException {
        	QuerySourceDictionary sourceDictionary = new QuerySourceDictionary();
            
            List visitors = new ArrayList();
            visitors.add(new WhereBracketAppender());
            visitors.add(new QuerySourceRegister(sourceDictionary));
            visitors.add(new EmptyWhereAppender());
            visitors.add(new DerivedTableToCrossJoinTransformer(sourceDictionary));
            visitors.add(new PathTransformer(sourceDictionary));
            visitors.add(new MemberOfTransformer(identityDefinerBuilder, sourceDictionary));
            visitors.add(new EmptyTransformer(sourceDictionary));
            visitors.add(new IsNullTransformer(sourceDictionary));
            visitors.add(new EntityReferenceTransformer(identityDefinerBuilder));
            visitors.add(new EmptyWhereTransformer());
            
            for (Iterator iter = visitors.iterator(); iter.hasNext();) {
                CommandTransform transform = (CommandTransform) iter.next();
                command = transform.transform(command);
            }
            
            return command;
        }
    }
    
}
