/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import org.tranql.cache.InTxCache;
import org.tranql.field.Row;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySourceDictionary;
import org.tranql.query.QueryCloner;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;

/**
 * Intermediate representation transformer.
 * 
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
 class InterRepresentationTransformer extends QueryCloner {    
    private final QuerySourceDictionary sourceDictionary;
    
    protected InterRepresentationTransformer(QuerySourceDictionary sourceDictionary) {
        this.sourceDictionary = sourceDictionary;
    }
    
    public QueryCommand transform(QueryCommand source) throws QueryException {
        Query targetQuery = (Query) (source.getQuery().visit(this, sourceDictionary));
        return new IRQueryCommand(targetQuery);
    }

    private static class IRQueryCommand implements QueryCommand {
        private final Query query;
        
        private IRQueryCommand(Query query) {
            this.query = query;
        }
        
        public Query getQuery() {
            return query;
        }

        public Object execute(InTxCache cache, ResultHandler handler, Row params, Object arg) throws QueryException {
            throw new UnsupportedOperationException();
        }
    }
}
