/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.field;


/**
 * Public accessor to a row for use at the end of a transform
 * pipeline.
 *
 * @version $Revision: 45 $ $Date: 2004-04-19 18:23:20 -0400 (Mon, 19 Apr 2004) $
 */
public class FieldAccessor implements FieldTransform {
    protected final Class fieldClass;
    protected final int slot;

    /**
     * Constructor specify the slot this is an accessor for.
     * @param slot the slot whose data this accessor manages
     * @param fieldClass the Class of the Objects the slot will contain
     */
    public FieldAccessor(int slot, Class fieldClass) {
        this.slot = slot;
        this.fieldClass = fieldClass;
    }

    public Class getFieldClass() {
        return fieldClass;
    }

    /**
     * Return a value from the supplied row.
     * @param row the row whose data should be returned
     * @return the value this row contains for this accessor, may be null
     */
    public Object get(Row row) {
        return row.get(slot);
    }

    /**
     * Set a value into the supplied row.
     * @param row the row whose data should be set
     * @param value the value to store in the row, may be null
     */
    public void set(Row row, Object value) {
        row.set(slot, value);
    }
}
