/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.identity;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheTable;
import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;

/**
 *
 *
 *
 * @version $Revision: 186 $ $Date: 2005-06-22 07:22:41 -0400 (Wed, 22 Jun 2005) $
 */
public class DerivedIdentity implements IdentityDefiner {
    private final CacheTable table;
    private final FieldTransform[] fieldTransforms;
    private final int[] slots;

    public DerivedIdentity(CacheTable table, FieldTransform[] fieldTransforms) {
        this.table = table;
        this.fieldTransforms = fieldTransforms;
        this.slots = null;
    }

    public DerivedIdentity(CacheTable table, int[] slots) {
        this.table = table;
        this.fieldTransforms = null;
        this.slots = slots;
    }

    public GlobalIdentity defineIdentity(Row row) throws UndefinedIdentityException {
        Object[] values;
        boolean allNull = true;
        if (null != fieldTransforms) {
            values = new Object[fieldTransforms.length];
            for (int i = 0; i < fieldTransforms.length; i++) {
                Object value;
                try {
                    value = fieldTransforms[i].get(row);
                } catch (FieldTransformException e) {
                    throw new UndefinedIdentityException("Cannot load value", e);
                }
                if (value == CacheRow.NO_DATA) {
                    throw new UndefinedIdentityException("Key value not loaded for field "+ fieldTransforms[i]);
                } else if (value != null) {
                    values[i] = value;
                    allNull = false;
                } else if (!allNull) {
                    throw new UndefinedIdentityException("Partial null key");
                }
            }
        } else {
            values = new Object[slots.length];
            for (int i = 0; i < slots.length; i++) {
                int slot = slots[i];
                Object value = row.get(slot);
                if (value == CacheRow.NO_DATA) {
                    throw new UndefinedIdentityException("Key value not loaded for slot "+ slot);
                } else if (value != null) {
                    values[i] = value;
                    allNull = false;
                } else if (!allNull) {
                    throw new UndefinedIdentityException("Partial null key");
                }
            }
        }
        return allNull ? null : new GlobalIdentity(table, new DerivedIdentityValue(values));
    }

    public Row extractIdentity(GlobalIdentity globalId) {
        Object[] values = ((DerivedIdentityValue) globalId.getId()).getValues();
        Row row = new Row(values);
        return row;
    }

    public void injectIdentity(CacheRow row) {
        GlobalIdentity globalId = row.getId();
        Object[] values = ((DerivedIdentityValue) globalId.getId()).getValues();
        for (int i = 0; i < slots.length; i++) {
            row.set(slots[i], values[i]);
        }
    }
}
