/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.identity;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheTable;
import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;

/**
 *
 *
 *
 * @version $Revision: 186 $ $Date: 2005-06-22 07:22:41 -0400 (Wed, 22 Jun 2005) $
 */
public class UserDefinedIdentity implements IdentityDefiner {
    private final CacheTable table;
    private final FieldTransform fieldTransform;
    private final int slot;

    public UserDefinedIdentity(CacheTable table, FieldTransform fieldTransform) {
        this.table = table;
        this.fieldTransform = fieldTransform;
        this.slot = -1;
    }

    public UserDefinedIdentity(CacheTable table, int slot) {
        this.table = table;
        this.fieldTransform = null;
        this.slot = slot;
    }

    public GlobalIdentity defineIdentity(Row row) throws UndefinedIdentityException {
        Object id;
        if (null != fieldTransform) {
            try {
                id = fieldTransform.get(row);
            } catch (FieldTransformException e) {
                throw new UndefinedIdentityException("Cannot load value", e);
            }
        } else {
            id = row.get(slot);
        }
        if (id == CacheRow.NO_DATA) {
            throw new UndefinedIdentityException("Key value is not loaded");
        }
        return id == null ? null : new GlobalIdentity(table, id);
    }

    public Row extractIdentity(GlobalIdentity globalId) {
        return new Row(new Object[] {globalId.getId()});
    }

    public void injectIdentity(CacheRow row) {
        GlobalIdentity globalId = row.getId();
        row.set(slot, globalId.getId());
    }
}
