/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.FaultHandler;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.ql.QueryException;

/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class CacheFaultHandler implements FaultHandler {
    private final FrontEndCache cache;
    private final FaultHandler next;
    private final int[] indices;
    
    public CacheFaultHandler(FrontEndCache cache, FaultHandler next, int[] indices) {
        this.cache = cache;
        this.next = next;
        this.indices = indices;
    }
    
    public void rowFault(InTxCache inTxCache, GlobalIdentity id) throws QueryException {
        try {
            CacheRow row = cache.get(id);
            if (null != row) {
                inTxCache.associate(row);
                return;
            }
            next.rowFault(inTxCache, id);
        } catch (AlreadyAssociatedException e) {
            throw (AssertionError) new AssertionError().initCause(e);
        } catch (CacheException e) {
            throw new QueryException(e);
        }
    }

    public void fieldFault(InTxCache inTxCache, CacheRow row) throws QueryException {
        try {
            boolean allFaulted = cache.get(row, indices);
            if (allFaulted) {
                return;
            }
            next.fieldFault(inTxCache, row);
            cache.set(row, indices);
        } catch (CacheException e) {
            throw new QueryException(e);
        }
    }
}