/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import org.tranql.cache.CacheRow;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;

/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
class CacheRowEntry extends Row {

    private static final Object NO_DATA = new Object() {
        public final String toString() {
            return "NO_DATA";
        }
    };

    protected final GlobalIdentity id;
    private boolean removed;

    public CacheRowEntry(CacheRow row) {
        super(new Object[row.getValues().length]);
        this.id = row.getId();
        for (int i = 0; i < values.length; i++) {
            if (row.isLoaded(i)) {
                values[i] = row.get(i);
            } else {
                values[i] = NO_DATA;
            }
        }
    }

    public void merge(CacheRow row) {
        for (int i = 0; i < values.length; i++) {
            if (row.isLoaded(i)) {
                values[i] = row.get(i);
            }
        }
    }

    public void merge(CacheRowEntry row) {
        for (int i = 0; i < values.length; i++) {
            if (row.isLoaded(i)) {
                values[i] = row.get(i);
            }
        }
    }

    public boolean isLoaded(int slot) {
        return values[slot] != NO_DATA;
    }

    public GlobalIdentity getId() {
        return id;
    }

    public void set(CacheRow row, int[] indices) {
        for (int i = 0; i < indices.length; i++) {
            int index = indices[i];
            if (row.isLoaded(index)) {
                values[index] = row.get(index);
            }
        }
    }

    public boolean get(CacheRow row, int[] indices) {
        boolean returned = true;
        for (int i = 0; i < indices.length; i++) {
            int index = indices[i];
            if (row.isLoaded(index)) {
                continue;
            } else if (isLoaded(index)) {
                row.set(index, values[index]);
            } else {
                returned = false;
            }
        }
        return returned;
    }
    
    public CacheRow buildCacheRow() {
        if (removed) {
            return null;
        }
        
        CacheRow row = id.getTable().emptyRow(id);
        for (int i = 0; i < values.length; i++) {
            if (isLoaded(i)) {
                row.set(i, values[i]);
            }
        }
        return row;
    }
    
    public void markRemoved() {
        removed = true;
    }
    
    public boolean isRemoved() {
        return removed;
    }
}