/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

import org.tranql.cache.CacheRow;
import org.tranql.identity.GlobalIdentity;

/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class FrontEndToCacheAdaptor implements FrontEndCache {
    private final TransactionManager tm;
    private final Cache cache;

    public FrontEndToCacheAdaptor(TransactionManager tm, Cache cache) {
        this.tm = tm;
        this.cache = cache;
    }
    
    public void put(CacheRow row) throws CacheException {
        Transaction tx = getTransaction();
        cache.put(tx, row);
    }

    public void set(CacheRow row, int[] indices) throws CacheException {
        Transaction tx = getTransaction();
        cache.set(tx, row, indices);
    }

    public CacheRow get(GlobalIdentity id) throws CacheException {
        Transaction tx = getTransaction();
        return cache.get(tx, id);
    }

    public boolean get(CacheRow target, int[] indices) throws CacheException {
        Transaction tx = getTransaction();
        return cache.get(tx, target, indices);
    }

    public void remove(GlobalIdentity id) throws CacheException {
        Transaction tx = getTransaction();
        cache.remove(tx, id);
    }

    private Transaction getTransaction() throws CacheException {
        try {
            return tm.getTransaction();
        } catch (SystemException e) {
            throw new CacheException(e);
        }
    }
}