/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import java.util.Map;

import org.tranql.cache.CacheFlushStrategy;
import org.tranql.cache.CacheRow;
import org.tranql.ql.QueryException;


/**
 *
 * @version $Revision: 344 $ $Date: 2006-06-04 08:09:49 -0400 (Sun, 04 Jun 2006) $
 */
public class InTxCacheTracker implements CacheFlushStrategy {
    private final FrontEndCache cache;
    private final CacheFlushStrategy delegate;
    
    public InTxCacheTracker(FrontEndCache cache, CacheFlushStrategy delegate) {
        this.cache = cache;
        this.delegate = delegate;
    }

    public void flush(Map cache) throws QueryException {
        delegate.flush(cache);
    }

    public void rowAssociated(CacheRow row) {
        delegate.rowAssociated(row);
        try {
            cache.put(row);
        } catch (CacheException e) {
            throw new RuntimeCacheException(e);
        }
    }
    
    public void rowAdded(CacheRow row) {
        delegate.rowAdded(row);
    }

    public void rowModified(CacheRow row) {
        delegate.rowModified(row);
    }

    public void rowRemoved(CacheRow row) {
        delegate.rowRemoved(row);
        try {
            cache.remove(row.getId());
        } catch (CacheException e) {
            throw new RuntimeCacheException(e);
        }
    }
}