/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.pkgenerator;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.BitSet;

import javax.sql.DataSource;

import org.tranql.cache.CacheRow;
import org.tranql.cache.DuplicateIdentityException;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.sql.jdbc.JDBCUtil;
import org.tranql.sql.jdbc.ResultBinding;

/**
 *
 * @version $Revision: 173 $ $Date: 2005-02-20 10:42:56 -0500 (Sun, 20 Feb 2005) $
 */
public class AutoIncrementTablePrimaryKeyGenerator implements PrimaryKeyGenerator {
    private final DataSource ds;
    private final String sqlText;
    private final ResultBinding binding;

    public AutoIncrementTablePrimaryKeyGenerator(DataSource ds, String sqlText, ResultBinding binding) {
        this.ds = ds;
        this.sqlText = sqlText;
        this.binding = binding;
    }

    public Object getNextPrimaryKey(CacheRow cacheRow) throws PrimaryKeyGeneratorException {
        Connection c = null;
        Statement s = null;
        ResultSet rs = null;
        try {
            c = ds.getConnection();
            s = c.createStatement();
            int rows = s.executeUpdate(sqlText, Statement.RETURN_GENERATED_KEYS);
            rs = s.getGeneratedKeys();
            if (!rs.next()) {
                throw new PrimaryKeyGeneratorException("SQL statement did not return an autogenerated key: " + sqlText);
            }
            return binding.getValue(rs);
        } catch (SQLException e) {
            throw new PrimaryKeyGeneratorException("Unable to generateQuery new identity value", e);
        } finally {
            JDBCUtil.close(c, s, rs);
        }
    }
    
    public CacheRow updateCache(InTxCache cache, GlobalIdentity id, CacheRow cacheRow) throws DuplicateIdentityException {
        CacheRow returned;
        Object[] values = cacheRow.getValues();
        BitSet modified = new BitSet(values.length);
        for (int i = 0; i < values.length; i++) {
            if ( cacheRow.isModified(i) ) {
                modified.set(i);
            }
        }
        returned = id.getTable().addRow(cache, id, cacheRow);
        returned.markClean();
        for (int i = 0; i < values.length; i++) {
            if ( modified.get(i) ) {
                returned.markDirty(i);
            }
        }
        return returned;
    }
}
