/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import org.tranql.schema.Entity;

/**
 * This abstract class associates and alias with a schema entity.
 *
 * This is used both to initally associate an entity with an alias
 * in an insert, update, delete or the from cluase of a select
 * query, and later it is used in the where clause to identify an
 * entity.
 *
 * This node expects no children.
 *
 * @version $Revision: 138 $ $Date: 2004-10-20 23:52:45 -0400 (Wed, 20 Oct 2004) $
 */
public abstract class AliasedEntity extends AbstractNode {
    private final Entity entity;
    private final String alias;

    public AliasedEntity(Entity entity, String alias) {
        this.entity = entity;
        this.alias = alias;
    }

    public Entity getEntity() {
        return entity;
    }

    public String getAlias() {
        return alias;
    }
}
