/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

/**
 * This class represents a conditional assignment in an update statement.
 * This is used to update an entity where all columns may not have been
 * loaded yet.  This node expects three children, column to set, a boolean
 * flag, and a value.  This style of update statement is more effieient
 * because the database can cache the prepared statement plan.  It also
 * allows us to have a single style of update statement which makes
 * replacement with a stored procedure easier.
 *
 * column = CASE WHEN flag THEN value ELSE column
 *
 * @version $Revision: 138 $ $Date: 2004-10-20 23:52:45 -0400 (Wed, 20 Oct 2004) $
 */
public class ConditionalAssignment extends AbstractNode {
    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
