/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;



/**
 * This node represents an aggregate function call.  This is used within a
 * select clause.  This node expects one child.
 *
 * @version $Revision: 173 $ $Date: 2005-02-21 02:42:56 +1100 (Mon, 21 Feb 2005) $
 */
public class CountAggregateFunction extends AggregateFunction {

    public static final FunctionType COUNT = new FunctionType() {
        public void checkType(Node node) {
            if (false == node instanceof Path &&
                    false == node instanceof EntityReference) {
                throw new MalformedNodeException("COUNT expect a path or an identification variable. Specified child is " + node);
            }
        }
        
        public Class getType(Node node) {
            return Integer.class;
        }

        public String getName() {
            return "COUNT";
        }
    };

    public CountAggregateFunction(boolean distinct) {
        super(COUNT, distinct);
    }

    public boolean isEntityReferenceCounted() {
        return getChild() instanceof EntityReference;
    }

    public EntityReference getEntityReference() {
        return (EntityReference) getChild();
    }
    
    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
    
}
