/**
 *
 *  Copyright 2004 Dain Sundstrom
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.io.Serializable;

/**
 * @version $Revision: 135 $ $Date: 2004-10-20 17:10:17 -0400 (Wed, 20 Oct 2004) $
 */
public final class DataType implements Serializable {

    // Be careful here.  If you change the ordinals, this class must be changed on evey client.
    private static int MAX_ORDINAL = 4;
    private static final DataType[] values = new DataType[MAX_ORDINAL + 1];
    public static final DataType BOOLEAN = new DataType("BOOLEAN", 0);
    public static final DataType STRING = new DataType("STRING", 1);
    public static final DataType NUMERIC = new DataType("NUMERIC", 2);
    public static final DataType DATETIME = new DataType("DATETIME", 3);
    public static final DataType ENTITY = new DataType("ENTITY", 4);

    private final transient String name;
    private final int ordinal;

    private DataType(String name, int ordinal) {
        assert ordinal < MAX_ORDINAL;
        assert values[ordinal] == null;
        this.name = name;
        this.ordinal = ordinal;
        values[ordinal] = this;
    }

    public String toString() {
        return name;
    }

    private Object readResolve() {
        return values[ordinal];
    }
}
