/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;


/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public final class SubQuerySource extends AliasedEntity {
    private final SubQuery subQuery;
    
    public SubQuerySource(SubQuery subQuery, String alias) {
        super(buildEntity(subQuery, alias), alias);
        this.subQuery = subQuery;
    }

    public SubQuery getSubQuery() {
        return subQuery;
    }
    
    private static Entity buildEntity(SubQuery subQuery, String alias) {
        SubQueryEntity entity = new SubQueryEntity(alias);
        
        Select select = (Select) subQuery.getChild();
        for (Node selectedChild = select.getChild(); null != selectedChild; selectedChild = selectedChild.getSibling()) {
            if (false == selectedChild instanceof AttributeReference) {
                throw new UnsupportedOperationException();
            }
            AttributeReference reference = (AttributeReference) selectedChild;
            entity.addAttribute(new SubQueryAttribute(reference.getAttribute()));
        }
        
        return entity;
    }

    private static class SubQueryEntity implements Entity {
        private final String name;
        private final List attributes = new ArrayList();
        private final Map attributeByName = new HashMap();
        
        private SubQueryEntity(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public String getPhysicalName() {
            return name;
        }

        protected void addAttribute(Attribute attribute) {
            attributes.add(attribute);
            attributeByName.put(attribute.getName(), attribute);
        }
        
        public Attribute getAttribute(String name) {
            return (Attribute) attributeByName.get(name);
        }

        public List getPrimaryKeyFields() {
            return Collections.EMPTY_LIST;
        }

        public List getAttributes() {
            return Collections.unmodifiableList(attributes);
        }

        public AssociationEnd getAssociationEnd(String name) {
            return null;
        }

        public AssociationEnd getAssociationEndDefiningFKAttribute(String fkAttribute) {
            return null;
        }

        public List getAssociationEnds() {
            return Collections.EMPTY_LIST;
        }
    }
    
    private static class SubQueryAttribute implements Attribute {
        private final Attribute attribute; 
        
        private SubQueryAttribute(Attribute attribute) {
            this.attribute = attribute;
        }
        
        public String getName() {
            return attribute.getName();
        }

        public String getPhysicalName() {
            return attribute.getPhysicalName();
        }

        public Class getType() {
            return attribute.getType();
        }

        public boolean isIdentity() {
            return false;
        }
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
