/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql.queryvisitor;

import org.tranql.ql.AggregateFunction;
import org.tranql.ql.Assignment;
import org.tranql.ql.AttributeReference;
import org.tranql.ql.Between;
import org.tranql.ql.BinaryOperation;
import org.tranql.ql.BooleanLiteral;
import org.tranql.ql.ConditionalAssignment;
import org.tranql.ql.CountAggregateFunction;
import org.tranql.ql.Delete;
import org.tranql.ql.DerivedTable;
import org.tranql.ql.Empty;
import org.tranql.ql.EntityReference;
import org.tranql.ql.Exists;
import org.tranql.ql.FKAttributeReference;
import org.tranql.ql.From;
import org.tranql.ql.Function;
import org.tranql.ql.In;
import org.tranql.ql.Insert;
import org.tranql.ql.IsNull;
import org.tranql.ql.Join;
import org.tranql.ql.Like;
import org.tranql.ql.MemberOf;
import org.tranql.ql.Not;
import org.tranql.ql.NumericLiteral;
import org.tranql.ql.OrderBy;
import org.tranql.ql.ParameterReference;
import org.tranql.ql.Path;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySource;
import org.tranql.ql.QueryVisitor;
import org.tranql.ql.RowConstructor;
import org.tranql.ql.Select;
import org.tranql.ql.SetList;
import org.tranql.ql.StringLiteral;
import org.tranql.ql.SubQuery;
import org.tranql.ql.SubQuerySource;
import org.tranql.ql.TableConstructor;
import org.tranql.ql.UnaryOperator;
import org.tranql.ql.Update;
import org.tranql.ql.Where;

/**
 *
 *
 * @version $Revision: 410 $ $Date: 2006-10-21 20:15:17 -0400 (Sat, 21 Oct 2006) $
 */
public class ExecutorVisitor implements QueryVisitor {
    protected final NodeExecutor executor;

    public ExecutorVisitor(NodeExecutor condition) {
        this.executor = condition;
    }

    public Object visit(Query node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Insert node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Update node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(SetList node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Assignment node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(ConditionalAssignment node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Delete node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Select node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(From node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Where node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(OrderBy node, Object param) throws QueryException {
        return executor.execute(node, param);
    }
    
    public Object visit(QuerySource node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(DerivedTable node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Join node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Path node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(EntityReference node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(AttributeReference node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(FKAttributeReference node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(TableConstructor node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(RowConstructor node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Function node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(AggregateFunction node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(CountAggregateFunction node, Object param) throws QueryException {
        return executor.execute(node, param);
    }
    
    public Object visit(BinaryOperation node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Not node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Between node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(In node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Like node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(Empty node, Object param) throws QueryException {
        return executor.execute(node, param);
    }
    
    public Object visit(Exists node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(IsNull node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(MemberOf node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(SubQuery node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(SubQuerySource node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(StringLiteral node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(NumericLiteral node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(BooleanLiteral node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

    public Object visit(ParameterReference node, Object param) throws QueryException {
        return executor.execute(node, param);
    }
    
    public Object visit(UnaryOperator node, Object param) throws QueryException {
        return executor.execute(node, param);
    }

}