/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.query;

import org.tranql.ql.QueryException;
import org.tranql.ql.Query;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;

/**
 *
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class ParamRemapper implements UpdateCommand {
    private final UpdateCommand next;
    private final FieldTransform[] transforms;

    public ParamRemapper(UpdateCommand next, FieldTransform[] transforms) {
        this.next = next;
        this.transforms = transforms;
    }

    public Query getQuery() {
        return next.getQuery();
    }

    public int execute(InTxCache cache, Row params) throws QueryException {
        Object[] newParams = new Object[transforms.length];
        for (int i = 0; i < newParams.length; i++) {
            try {
                newParams[i] = transforms[i].get(params);
            } catch (FieldTransformException e) {
                throw new QueryException("Unable to transform parameter "+ i, e);
            }
        }
        return next.execute(cache, new Row(newParams));
    }
}
