/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.schema;

import java.io.Serializable;
import java.util.LinkedHashMap;

/**
 *
 *
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public interface Association extends Serializable {
    Entity getManyToManyEntity();

    AssociationEnd getOtherEnd(AssociationEnd end);
    
    AssociationEnd[] getAssociationEnds();
    
    JoinDefinition getJoinDefinition();
    
    JoinDefinition getLeftJoinDefinition();
    
    JoinDefinition getRightJoinDefinition();
    
    public class JoinDefinition implements Serializable {
        private final Entity pkEntity;
        private final Entity fkEntity;
        private final LinkedHashMap pkToFkMapping;

        public JoinDefinition(Entity pkEntity, Entity fkEntity, LinkedHashMap pkToFkMapping) {
            this.pkEntity = pkEntity;
            this.fkEntity = fkEntity;
            this.pkToFkMapping = new LinkedHashMap(pkToFkMapping);
        }
        
        public Entity getFKEntity() {
            return fkEntity;
        }
        
        public Entity getPKEntity() {
            return pkEntity;
        }
        
        public LinkedHashMap getPKToFKMapping() {
            return new LinkedHashMap(pkToFkMapping);
        }
    }
}
