/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.schema;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 *
 *
 *
 * @version $Revision: 246 $ $Date: 2005-08-26 22:09:42 -0400 (Fri, 26 Aug 2005) $
 */
public class EntityComparator implements Comparator {

    public int compare(Object o1, Object o2) {
        if (false == o1 instanceof Entity) {
            throw new IllegalArgumentException();
        } else if (false == o1 instanceof Entity) {
            throw new IllegalArgumentException();
        }
        Entity entity1 = (Entity) o1;
        Entity entity2 = (Entity) o2;
        
        if (entity1 == entity2) {
            return 0;
        }
        
        boolean reference = doesReference(entity1, entity2, new ArrayList());
        if (reference) {
            return 1;
        }
        
        reference = doesReference(entity2, entity1, new ArrayList());
        if (reference) {
            return -1;
        }
        
        return 1;
    }
    
    private boolean doesReference(Entity rootEntity, Entity stopEntity, Collection traversed) {
        if (traversed.contains(rootEntity)) {
            return false;
        }
        traversed.add(rootEntity);
        
        List ends = rootEntity.getAssociationEnds();
        for (Iterator iter = ends.iterator(); iter.hasNext();) {
            AssociationEnd end = (AssociationEnd) iter.next();
            if (false == end.isOnPKSide()) {
                Entity relatedEntity = end.getEntity();
                if (stopEntity == relatedEntity) {
                    return true;
                }
                boolean reference = doesReference(relatedEntity, stopEntity, traversed);
                if (reference) {
                    return true;
                }
            }
        }
        return false;
    }
}
