/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql;

import javax.sql.DataSource;

import org.tranql.abstractschema.AbstractSchema;
import org.tranql.query.CommandFactory;
import org.tranql.sql.prefetch.PrefetchGroupDictionary;

/**
 * 
 * 
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public abstract class SQLSchema extends AbstractSchema {
    private final DataSource ds;
    private final transient EJBQLCompilerFactory compilerFactory;
    private final transient PrefetchGroupDictionary groupDictionary;
    
    public SQLSchema(String name, DataSource ds, CommandFactory commandFactory, EJBQLCompilerFactory compilerFactory) {
        super(name, name + "[SQL Domain]", commandFactory);
        this.ds = ds;
        this.compilerFactory = compilerFactory;
        this.groupDictionary = new PrefetchGroupDictionary(this);
    }

    public DataSource getDataSource() {
        return ds;
    }

    public EJBQLCompilerFactory getEJBQLCompilerFactory() {
        return compilerFactory;
    }

    public PrefetchGroupDictionary getGroupDictionary() {
        return groupDictionary;
    }

    public void addTable(Table table) {
        addEntity(table);
    }

    public Table getTable(String tableName) {
        return (Table) getEntity(tableName);
    }
}
