/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;

import org.apache.commons.logging.LogFactory;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.query.UpdateCommand;

/**
 *
 *
 *
 * @version $Revision: 432 $ $Date: 2006-12-12 00:17:36 -0500 (Tue, 12 Dec 2006) $
 */
public class JDBCUpdateCommand implements UpdateCommand {
	private final static org.apache.commons.logging.Log log = LogFactory.getLog(JDBCUpdateCommand.class);

    private final Query query;
    private final DataSource ds;
    private final String sqlText;
    private final InputBinding[] paramBindings;

    public JDBCUpdateCommand(Query query, DataSource ds, String sql, InputBinding[] inBindings) {
        this.query = query;
        this.ds = ds;
        this.sqlText = sql;
        this.paramBindings = inBindings;
    }

    public Query getQuery() {
        return query;
    }

    public String getSQLText() {
        return sqlText;
    }

    public int execute(InTxCache cache, Row row) throws QueryException {
        Connection c = null;
        PreparedStatement ps = null;
        try {
            c = ds.getConnection();
            ps = c.prepareStatement(sqlText);
            
            if (log.isTraceEnabled())
                log.trace("Prepared SQL Statement=\""+sqlText+"\"");
            
            for (int i = 0; i < paramBindings.length; i++) {
                paramBindings[i].setValue(ps, row);
                if (log.isTraceEnabled()) {
                	Object o = row.get(i);
                    if (o==null) 
                        log.trace("  Setting parm["+i+"]=(null)");
                    else
                        log.trace("  Setting parm["+i+"]="+o.toString());
                }
            }
            return ps.executeUpdate();
        } catch (SQLException e) {
            throw new QueryException("Error executing statement: " + sqlText, e);
        } catch (FieldTransformException e) {
            throw new QueryException(e);
        } finally {
            JDBCUtil.close(c, ps);
        }
    }
}
