/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import javax.sql.DataSource;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheTable;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.identity.UndefinedIdentityException;

/**
 *
 *
 *
 * @version $Revision: 47 $ $Date: 2004-04-20 14:37:15 -0400 (Tue, 20 Apr 2004) $
 */
public class SQLIdentityGenerator implements IdentityDefiner {
    private final DataSource ds;
    private final CacheTable table;
    private final String sqlText;
    private final ResultBinding binding;

    public SQLIdentityGenerator(DataSource ds, CacheTable table, String sqlText, ResultBinding binding) {
        this.ds = ds;
        this.table = table;
        this.sqlText = sqlText;
        this.binding = binding;
    }

    public GlobalIdentity defineIdentity(Row row) throws UndefinedIdentityException {
        Connection c = null;
        Statement s = null;
        ResultSet rs = null;
        try {
            c = ds.getConnection();
            s = c.createStatement();
            rs = s.executeQuery(sqlText);
            if (!rs.next()) {
                throw new UndefinedIdentityException("SQL statement did not return a result: " + sqlText);
            }
            return new GlobalIdentity(table, binding.getValue(rs));
        } catch (SQLException e) {
            throw new UndefinedIdentityException("Unable to generateQuery new identity value", e);
        } finally {
            JDBCUtil.close(c, s, rs);
        }
    }

    public Row extractIdentity(GlobalIdentity globalId) {
        return new Row(new Object[]{globalId.getId()});
    }

    public void injectIdentity(CacheRow row) {
        // nothing to do... row values do not contain the identity
    }
}
