/**
 *
 *  Copyright 2005  Matt Hogstrom
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.tranql.ql.QueryBinding;

/**
 *
 *
 *
 * @version $Revision: 187 $ $Date: 2005-06-22 22:44:04 -0400 (Wed, 22 Jun 2005) $
 */
public class BigDecimalBinding extends AbstractBinding {
    public BigDecimalBinding(int index, QueryBinding transform) {
        super(index, transform);
    }

    public Object getValue(ResultSet rs) throws SQLException {
        BigDecimal value = rs.getBigDecimal(index);
        return rs.wasNull() ? null : value;
    }

    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        if (value == null) {
            ps.setNull(index, getSQLType());
        } else {
            ps.setBigDecimal(index, (BigDecimal) value);
        }
    }

    public void register(CallableStatement cs) throws SQLException {
        cs.registerOutParameter(index, getSQLType());
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        BigDecimal value = cs.getBigDecimal(index);
        return cs.wasNull() ? null : value;
    }
    
    public int getSQLType() {
        return Types.FLOAT;
    }
}
