/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.ql.QueryBinding;
import org.tranql.sql.jdbc.LOBBinding;

/**
 *
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public class BlobBinding extends AbstractBinding implements LOBBinding {
    public BlobBinding(int index, QueryBinding transform) {
        super(index, transform);
    }

    public Object getValue(ResultSet rs) throws SQLException {
        return rs.getBinaryStream(index);
    }

    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        if (value == null) {
            ps.setBinaryStream(index, null, 0);
        } else {
            InputStream in = (InputStream) value;
            int length;
            try {
                ps.setBinaryStream(index, in, in.available());
            } catch (IOException e) {
                throw (SQLException) new SQLException().initCause(e);
            }
        }
    }
    
    public void register(CallableStatement cs) throws SQLException {
        cs.registerOutParameter(index, getSQLType());
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        return cs.getBlob(index);
    }

    public void setValue(Object lob, Object value) throws SQLException {
        Blob blob = (Blob) lob;
        InputStream in = (InputStream) value;
        
        if (null == in) {
            blob.truncate(1);
        }

        OutputStream out = blob.setBinaryStream(1);
        byte[] buffer = new byte[1024];
        int read;
        long overall = 0;
        try {
            while (-1 != (read = in.read(buffer))) {
                out.write(buffer, 0, read);
                overall += read;
            }
        } catch (IOException e) {
            throw (SQLException) new SQLException().initCause(e);
        }
        if (blob.length() > overall) {
            blob.truncate(overall);
        }
    }

    public void setValue(Object value, Row row) throws SQLException, FieldTransformException {
        setValue(transform.get(row), value);
    }
    
    public int getSQLType() {
        return Types.BLOB;
    }
}
