/**
 * 
 */
package org.tranql.sql.prefetch;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.EmptySlotLoader;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.identity.UndefinedIdentityException;
import org.tranql.ql.QueryException;
import org.tranql.query.ResultHandler;
import org.tranql.sql.prefetch.PrefetchGroupHandler.CacheContext;

class TopHandler implements ResultHandler, Serializable {
    private transient final ResultHandler topHandler;
    private final IdentityDefiner idDefiner;
    private final EmptySlotLoader[] slotLoaders;
    private final Collection childHandlers;
    
    public TopHandler(ResultHandler topHandler,
            IdentityDefiner idDefiner,
            EmptySlotLoader[] slotLoaders,
            Collection childHandlers) {
        this.topHandler = topHandler;
        this.idDefiner = idDefiner;
        this.slotLoaders = slotLoaders;
        this.childHandlers = childHandlers;
    }
    
    public Object fetched(Row row, Object arg) throws QueryException {
        CacheContext ctx = (CacheContext) arg;
        InTxCache cache = ctx.cache;
        CacheRow parentRow;
        GlobalIdentity id;
        try {
            id = idDefiner.defineIdentity(row);
            if (null == id) {
                ctx.nestedContext = topHandler.fetched(row, ctx.nestedContext);
                ctx.lastTopId = id;
                return arg;
            }
            parentRow = cache.get(id);

            boolean addToCache = false;
            if (null == parentRow) {
                addToCache = true;
                parentRow = id.getTable().emptyRow(id);
            }

            for (int i=0; i < slotLoaders.length; i++) {
                slotLoaders[i].set(parentRow, row);
            }

            if (addToCache) {
                try {
                    cache.add(parentRow);
                } catch (AlreadyAssociatedException e) {
                    throw new AssertionError();
                }
            }
            
            if (null == ctx.lastTopId || false == ctx.lastTopId.equals(id)) {
                ctx.nestedContext = topHandler.fetched(row, ctx.nestedContext);
            }
            ctx.lastTopId = id;
        } catch (FieldTransformException e) {
            throw new QueryException(e);
        } catch (UndefinedIdentityException e) {
            throw new QueryException(e);
        }

        ctx.cacheRow = parentRow;
        for (Iterator iter = childHandlers.iterator(); iter.hasNext();) {
            ResultHandler handler = (ResultHandler) iter.next();
            handler.fetched(row, ctx);
        }
        ctx.cacheRow = null;

        return arg;
    }

    public Object endFetched(Object arg) throws QueryException {
        return arg;
    }
}