/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.typeconverter;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamClass;

import org.tranql.sql.TypeConversionException;
import org.tranql.sql.TypeConverter;


/**
 *
 *
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public final class SerializableConverter implements TypeConverter {
    public static final SerializableConverter CONVERTER = new SerializableConverter();

    public Object convertJavaToSQLType(Object value) throws TypeConversionException {
        ByteArrayOutputStream memOut = new ByteArrayOutputStream();
        try {
            ObjectOutputStream out = new ObjectOutputStream(memOut);
            out.writeObject(value);
            out.close();
        } catch (IOException e) {
            throw new TypeConversionException(e);
        }
        return new ByteArrayInputStream(memOut.toByteArray());
    }

    public Object convertSQLToJavaType(Object value) throws TypeConversionException {
        InputStream valueIN = (InputStream) value;
        if (null == valueIN) {
            return null;
        }
        try {
            ObjectInputStream in = new TCCLObjectInputStream(valueIN);
            return in.readObject();
        } catch (Exception e) {
            throw new TypeConversionException(e);
        } finally {
            try {
                if (null != valueIN) {
                    valueIN.close();
                }
            } catch (IOException e) {
                throw new TypeConversionException(e);
            }
        }
    }
    
    private static class TCCLObjectInputStream extends ObjectInputStream {
        private final ClassLoader classLoader;
        
        protected TCCLObjectInputStream(InputStream in) throws IOException, SecurityException {
            super(in);
            this.classLoader = Thread.currentThread().getContextClassLoader();
        }

        protected Class resolveClass(ObjectStreamClass classDesc) throws IOException, ClassNotFoundException {
            return classLoader.loadClass(classDesc.getName());
        }
    }
}
